@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2019-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -74,7 +74,7 @@ protected:
                         }
                     }
 
-                    // Apparently we're looking for one that does not prexist.
+                    // Apparently we're looking for a new one.
                     return (TestConfigBackendDHCPv4Ptr(new TestConfigBackendDHCPv4(params)));
                 });
     }
@@ -264,18 +264,15 @@ TEST_F(Dhcp4CBTest, mergeOptionDefs) {
 
     // Create option one replacement and add it to first backend.
     OptionDefinitionPtr def;
-    def.reset(new OptionDefinition("one", 101, "uint16"));
-    def->setOptionSpaceName("isc");
+    def.reset(new OptionDefinition("one", 101, "isc", "uint16"));
     db1_->createUpdateOptionDef4(ServerSelector::ALL(), def);
 
     // Create option three and add it to first backend.
-    def.reset(new OptionDefinition("three", 3, "string"));
-    def->setOptionSpaceName("isc");
+    def.reset(new OptionDefinition("three", 3, "isc", "string"));
     db1_->createUpdateOptionDef4(ServerSelector::ALL(), def);
 
     // Create option four and add it to second backend.
-    def.reset(new OptionDefinition("four", 4, "string"));
-    def->setOptionSpaceName("isc");
+    def.reset(new OptionDefinition("four", 4, "isc", "string"));
     db2_->createUpdateOptionDef4(ServerSelector::ALL(), def);
 
     // Should parse and merge without error.
@@ -340,21 +337,24 @@ TEST_F(Dhcp4CBTest, mergeOptions) {
     // Add host-name to the first backend.
     opt.reset(new OptionDescriptor(
               createOption<OptionString>(Option::V4, DHO_HOST_NAME,
-                                         true, false, "new.example.com")));
+                                         true, false, false,
+                                         "new.example.com")));
     opt->space_name_ = DHCP4_OPTION_SPACE;
     db1_->createUpdateOption4(ServerSelector::ALL(), opt);
 
     // Add boot-file-name to the first backend.
     opt.reset(new OptionDescriptor(
               createOption<OptionString>(Option::V4, DHO_BOOT_FILE_NAME,
-                                         true, false, "my-boot-file")));
+                                         true, false, false,
+                                         "my-boot-file")));
     opt->space_name_ = DHCP4_OPTION_SPACE;
     db1_->createUpdateOption4(ServerSelector::ALL(), opt);
 
     // Add boot-file-name to the second backend.
     opt.reset(new OptionDescriptor(
               createOption<OptionString>(Option::V4, DHO_BOOT_FILE_NAME,
-                                         true, false, "your-boot-file")));
+                                         true, false, false,
+                                         "your-boot-file")));
     opt->space_name_ = DHCP4_OPTION_SPACE;
     db2_->createUpdateOption4(ServerSelector::ALL(), opt);
 
@@ -490,19 +490,19 @@ TEST_F(Dhcp4CBTest, mergeSubnets) {
     SrvConfigPtr staging_cfg = CfgMgr::instance().getStagingCfg();
 
     CfgSubnets4Ptr subnets = staging_cfg->getCfgSubnets4();
-    Subnet4Ptr staged_subnet;
+    ConstSubnet4Ptr staged_subnet;
 
     // Subnet1 should have been added from db1 config
-    staged_subnet = subnets->getSubnet(1);
+    staged_subnet = subnets->getBySubnetId(1);
     ASSERT_TRUE(staged_subnet);
 
     // Subnet2 should have come from the json config
-    staged_subnet = subnets->getSubnet(2);
+    staged_subnet = subnets->getBySubnetId(2);
     ASSERT_TRUE(staged_subnet);
 
     // Subnet3, which is in db2 should not have been merged, since it is
     // first found, first used?
-    staged_subnet = subnets->getSubnet(3);
+    staged_subnet = subnets->getBySubnetId(3);
     ASSERT_FALSE(staged_subnet);
 }
 
