@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,7 +11,7 @@
 #include <asiolink/asiolink.h>
 #include <cc/data.h>
 #include <cc/command_interpreter.h>
-#include <database/database_connection.h>
+#include <util/reconnect_ctl.h>
 #include <dhcpsrv/timer_mgr.h>
 #include <dhcp6/dhcp6_srv.h>
 
@@ -80,11 +80,18 @@ public:
     /// in them.
     ///
     /// Currently supported commands are:
-    /// - config-reload
-    /// - config-test
     /// - shutdown
     /// - libreload
+    /// - config-reload
+    /// - config-set
+    /// - config-get
+    /// - config-test
+    /// - dhcp-disable
+    /// - dhcp-enable
+    /// - version-get
+    /// - build-report
     /// - leases-reclaim
+    /// - config-write
     /// ...
     ///
     /// @note It never throws.
@@ -105,21 +112,32 @@ public:
     /// As pointer to this method is used a callback in ASIO used in
     /// ModuleCCSession, it has to be static.
     ///
-    /// @param new_config textual representation of the new configuration
+    /// @param config textual representation of the new configuration
     ///
     /// @return status of the config update
     static isc::data::ConstElementPtr
-    processConfig(isc::data::ConstElementPtr new_config);
+    processConfig(isc::data::ConstElementPtr config);
 
     /// @brief Configuration checker
     ///
     /// This is a method for checking incoming configuration.
     ///
-    /// @param new_config JSON representation of the new configuration
+    /// @param config JSON representation of the new configuration
     ///
     /// @return status of the config check
-    isc::data::ConstElementPtr
-    checkConfig(isc::data::ConstElementPtr new_config);
+    static isc::data::ConstElementPtr
+    checkConfig(isc::data::ConstElementPtr config);
+
+    /// @brief Configuration checker for hook libraries
+    ///
+    /// This is a method for checking incoming configuration in the hooks
+    /// libraries. It calls dhcp4_srv_configured hook point for all hooks.
+    ///
+    /// @param config JSON representation of the new configuration
+    ///
+    /// @return status of the config check
+    static isc::data::ConstElementPtr
+    finishConfigHookLibraries(isc::data::ConstElementPtr config);
 
     /// @brief Returns pointer to the sole instance of Dhcpv6Srv
     ///
@@ -129,7 +147,6 @@ public:
     }
 
 private:
-
     /// @brief Callback that will be called from iface_mgr when data
     /// is received over control socket.
     ///
@@ -234,7 +251,7 @@ private:
     /// @brief A handler for processing 'dhcp-disable' command.
     ///
     /// @param command command name (ignored).
-    /// @param args aguments for the command. It must be a map and
+    /// @param args arguments for the command. It must be a map and
     /// it may include optional 'max-period' parameter.
     ///
     /// @return result of the command.
@@ -387,7 +404,8 @@ private:
     /// deleted.
     void deleteExpiredReclaimedLeases(const uint32_t secs);
 
-    /// @brief Callback DB backends should invoke upon loss of the connectivity
+    /// @brief Callback DB backends should be invoked upon loss of the
+    /// connectivity.
     ///
     /// This function is invoked by DB backends when they detect a loss of
     /// connectivity.  The parameter, db_reconnect_ctl, conveys the configured
@@ -403,26 +421,41 @@ private:
     /// configured reconnect parameters
     ///
     /// @return false if reconnect is not configured, true otherwise
-    bool dbLostCallback(db::ReconnectCtlPtr db_reconnect_ctl);
+    bool dbLostCallback(util::ReconnectCtlPtr db_reconnect_ctl);
 
-    /// @brief Callback DB backends should invoke upon restoration of
-    /// connectivity
+    /// @brief Callback DB backends should be invoked upon restoration of
+    /// connectivity.
     ///
     /// This function is invoked by DB backends when they recover the
     /// connectivity. It starts the DHCP service after the connection is
     /// recovered.
     ///
+    /// @param db_reconnect_ctl pointer to the ReconnectCtl containing the
+    /// configured reconnect parameters
+    ///
     /// @return false if reconnect is not configured, true otherwise
-    bool dbRecoveredCallback(db::ReconnectCtlPtr db_reconnect_ctl);
+    bool dbRecoveredCallback(util::ReconnectCtlPtr db_reconnect_ctl);
 
-    /// @brief Callback DB backends should invoke upon failing to restore
-    /// connectivity
+    /// @brief Callback DB backends should be invoked upon failing to restore
+    /// connectivity.
     ///
     /// This function is invoked by DB backends when they fail to recover the
     /// connectivity. It stops the server.
     ///
+    /// @param db_reconnect_ctl pointer to the ReconnectCtl containing the
+    /// configured reconnect parameters
+    ///
     /// @return false if reconnect is not configured, true otherwise
-    bool dbFailedCallback(db::ReconnectCtlPtr db_reconnect_ctl);
+    bool dbFailedCallback(util::ReconnectCtlPtr db_reconnect_ctl);
+
+    /// @brief This callback should be invoked upon failing to bind sockets.
+    ///
+    /// This function is invoked during the configuration of the interfaces
+    /// when they fail to bind the service sockets. It may stop the server.
+    ///
+    /// @param reconnect_ctl pointer to the ReconnectCtl containing the
+    /// configured reconnect parameters
+    void openSocketsFailedCallback(util::ReconnectCtlPtr reconnect_ctl);
 
     /// @brief Callback invoked periodically to fetch configuration updates
     /// from the Config Backends.
