@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -798,7 +798,7 @@ TEST_F(RebindTest, directClientPDChangingPrefix) {
     // but the prefix will not match. According to the RFC 8415, section 18.3.5
     // the server may return delegated prefix with lifetime of 0 when it
     // finds that the lease entry for the particular IAID but the prefix
-    // is not appropriate. This constitues explicit notification to the
+    // is not appropriate. This constitutes explicit notification to the
     // client to not use this prefix.
     ASSERT_NO_THROW(client.doRebind());
     // Make sure that the server has discarded client's message. In such case,
@@ -1068,8 +1068,7 @@ TEST_F(RebindTest, docsisORO) {
     opt = client.config_.findOption(D6O_VENDOR_OPTS);
     ASSERT_TRUE(opt);
     // The vendor option must be a OptionVendor object.
-    boost::shared_ptr<OptionVendor> vendor =
-        boost::dynamic_pointer_cast<OptionVendor>(opt);
+    OptionVendorPtr vendor = boost::dynamic_pointer_cast<OptionVendor>(opt);
     ASSERT_TRUE(vendor);
     // The vendor-id should be DOCSIS.
     EXPECT_EQ(VENDOR_ID_CABLE_LABS, vendor->getVendorId());
