@@ -9,16 +9,15 @@
 
 #include <config.h>
 
+#include <cc/command_interpreter.h>
 #include <netconf/netconf.h>
 #include <netconf/netconf_controller.h>
 #include <netconf/netconf_log.h>
-#include <cc/command_interpreter.h>
 #include <yang/translator_config.h>
 #include <yang/yang_revisions.h>
 
-#include <boost/algorithm/string.hpp>
-
 #include <sstream>
+#include <vector>
 
 using namespace std;
 using namespace isc::config;
@@ -136,9 +135,9 @@ public:
             .arg(service_pair_.first)
             .arg(tree);
     }
-};
+};  // NetconfAgentCallback
 
-} // end of anonymous namespace
+}  //anonymous namespace
 
 namespace isc {
 namespace netconf {
@@ -170,7 +169,7 @@ NetconfAgent::init(NetconfCfgMgrPtr cfg_mgr) {
 
     for (auto const& pair : *servers) {
         yangConfig(pair);
-        subscribeConfig(pair);
+        subscribeToDataChanges(pair);
         subscribeToNotifications(pair);
     }
 }
@@ -184,22 +183,25 @@ NetconfAgent::clear() {
 
 void
 NetconfAgent::keaConfig(const CfgServersMapPair& service_pair) {
+    string const& server(service_pair.first);
+    CfgServerPtr const& configuration(service_pair.second);
+
     // If the boot-update flag is not set.
-    if (!service_pair.second->getBootUpdate()) {
+    if (!configuration->getBootUpdate()) {
         return;
     }
-    CfgControlSocketPtr ctrl_sock = service_pair.second->getCfgControlSocket();
+    CfgControlSocketPtr ctrl_sock = configuration->getCfgControlSocket();
     if (!ctrl_sock) {
         return;
     }
     ControlSocketBasePtr comm;
     try {
         comm = controlSocketFactory(ctrl_sock);
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         ostringstream msg;
         msg << "createControlSocket failed with " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_GET_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return;
     }
@@ -207,15 +209,15 @@ NetconfAgent::keaConfig(const CfgServersMapPair& service_pair) {
     int rcode;
     ConstElementPtr config;
     LOG_INFO(netconf_logger, NETCONF_GET_CONFIG_STARTED)
-        .arg(service_pair.first);
+        .arg(server);
     try {
-        answer = comm->configGet(service_pair.first);
+        answer = comm->configGet(server);
         config = parseAnswer(rcode, answer);
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         ostringstream msg;
         msg << "config-get command failed with " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_GET_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return;
     }
@@ -223,19 +225,19 @@ NetconfAgent::keaConfig(const CfgServersMapPair& service_pair) {
         ostringstream msg;
         msg << "config-get command returned " << answerToText(answer);
         LOG_ERROR(netconf_logger, NETCONF_GET_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return;
     }
     if (!config) {
         LOG_ERROR(netconf_logger, NETCONF_GET_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg("config-get command returned an empty configuration");
         return;
     }
     LOG_DEBUG(netconf_logger, NETCONF_DBG_TRACE_DETAIL_DATA,
               NETCONF_GET_CONFIG)
-        .arg(service_pair.first)
+        .arg(server)
         .arg(prettyPrint(config));
 }
 
@@ -246,7 +248,7 @@ NetconfAgent::initSysrepo() {
         running_sess_->switchDatastore(Datastore::Running);
         startup_sess_ = Connection{}.sessionStart();
         startup_sess_->switchDatastore(Datastore::Startup);
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         isc_throw(Unexpected, "Can't establish a sysrepo session: "
                   << ex.what());
     }
@@ -334,71 +336,74 @@ NetconfAgent::checkModules(CfgServersMapPtr const& servers /* = {} */) const {
 
 void
 NetconfAgent::yangConfig(const CfgServersMapPair& service_pair) {
+    string const& server(service_pair.first);
+    CfgServerPtr const& configuration(service_pair.second);
+
     // If we're shutting down, or the boot-update flag is not set or the model
     // associated with it is not specified.
-    if (!service_pair.second->getBootUpdate() ||
-        service_pair.second->getModel().empty()) {
+    if (!configuration->getBootUpdate() ||
+        configuration->getModel().empty()) {
         return;
     }
 
     // First we need a way to reach the actual servers.
-    CfgControlSocketPtr ctrl_sock = service_pair.second->getCfgControlSocket();
+    CfgControlSocketPtr ctrl_sock = configuration->getCfgControlSocket();
     if (!ctrl_sock) {
         return;
     }
 
     LOG_INFO(netconf_logger, NETCONF_SET_CONFIG_STARTED)
-        .arg(service_pair.first);
-    ConstElementPtr config;
+        .arg(server);
+    ElementPtr config;
     try {
         // Retrieve configuration from Sysrepo.
-        TranslatorConfig tc(*startup_sess_, service_pair.second->getModel());
+        TranslatorConfig tc(*startup_sess_, configuration->getModel());
         config = tc.getConfig();
         if (!config) {
             ostringstream msg;
             msg << "YANG configuration for "
-                << service_pair.second->getModel()
+                << configuration->getModel()
                 << " is empty";
             LOG_ERROR(netconf_logger, NETCONF_SET_CONFIG_FAILED)
-                .arg(service_pair.first)
+                .arg(server)
                 .arg(msg.str());
             return;
         } else {
             LOG_DEBUG(netconf_logger, NETCONF_DBG_TRACE_DETAIL_DATA,
                       NETCONF_SET_CONFIG)
-                .arg(service_pair.first)
+                .arg(server)
                 .arg(prettyPrint(config));
         }
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         ostringstream msg;
-        msg << "get YANG configuration for " << service_pair.first
+        msg << "get YANG configuration for " << server
             << " failed with " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_SET_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return;
     }
     ControlSocketBasePtr comm;
     try {
         comm = controlSocketFactory(ctrl_sock);
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         ostringstream msg;
         msg << "control socket creation failed with " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_SET_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return;
     }
     ConstElementPtr answer;
     int rcode;
     try {
-        answer = comm->configSet(config, service_pair.first);
+        answer = comm->configSet(config, server);
         parseAnswer(rcode, answer);
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         ostringstream msg;
         msg << "config-set command failed with " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_SET_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return;
     }
@@ -406,26 +411,27 @@ NetconfAgent::yangConfig(const CfgServersMapPair& service_pair) {
         ostringstream msg;
         msg << "config-set command returned " << answerToText(answer);
         LOG_ERROR(netconf_logger, NETCONF_SET_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return;
     }
     LOG_INFO(netconf_logger, NETCONF_BOOT_UPDATE_COMPLETED)
-        .arg(service_pair.first);
+        .arg(server);
 }
 
 void
-NetconfAgent::subscribeConfig(const CfgServersMapPair& service_pair) {
-    string const& model(service_pair.second->getModel());
+NetconfAgent::subscribeToDataChanges(const CfgServersMapPair& service_pair) {
+    string const& server(service_pair.first);
+    CfgServerPtr const& configuration(service_pair.second);
+    string const& model(configuration->getModel());
 
     // If we're shutting down, or the subscribe-changes flag is not set or
-    // the model associated with it is not specified.
-    if (!service_pair.second->getSubscribeChanges() ||
-        model.empty()) {
+    // the model is not specified, give up on subscribing.
+    if (!configuration->getSubscribeChanges() || model.empty()) {
         return;
     }
     LOG_INFO(netconf_logger, NETCONF_SUBSCRIBE_CONFIG)
-        .arg(service_pair.first)
+        .arg(server)
         .arg(model);
     auto callback = [=](Session session,
                         uint32_t subscription_id,
@@ -438,37 +444,38 @@ NetconfAgent::subscribeConfig(const CfgServersMapPair& service_pair) {
     };
     try {
         SubscribeOptions options(SubscribeOptions::Default);
-        if (!service_pair.second->getValidateChanges()) {
+        if (!configuration->getValidateChanges()) {
             options = options | SubscribeOptions::DoneOnly;
         }
         Subscription subscription(
-            running_sess_->onModuleChange(model, callback, std::nullopt, 0, options));
-        subscriptions_.emplace(service_pair.first, forward<Subscription>(subscription));
-    } catch (const std::exception& ex) {
+            running_sess_->onModuleChange(model, callback, nullopt, 0, options));
+        subscriptions_.emplace(server, forward<Subscription>(subscription));
+    } catch (exception const& ex) {
         ostringstream msg;
         msg << "module change subscribe failed with " << ex.what();
         msg << "change subscription for model " << model <<
             " failed with: " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_SUBSCRIBE_CONFIG_FAILED)
-            .arg(service_pair.first)
-            .arg(service_pair.second->getModel())
+            .arg(server)
+            .arg(configuration->getModel())
             .arg(msg.str());
         return;
     }
 }
 
-
 void
 NetconfAgent::subscribeToNotifications(const CfgServersMapPair& service_pair) {
-    std::string const& model(service_pair.second->getModel());
+    string const& server(service_pair.first);
+    CfgServerPtr const& configuration(service_pair.second);
+    string const& model(configuration->getModel());
+
     // If we're shutting down, or the subscribe-changes flag is not set or
-    // the model associated with it is not specified.
-    if (!service_pair.second->getSubscribeNotifications() ||
-        model.empty()) {
+    // the model is not specified, give up on subscribing.
+    if (!configuration->getSubscribeNotifications() || model.empty()) {
         return;
     }
     LOG_INFO(netconf_logger, NETCONF_SUBSCRIBE_NOTIFICATIONS)
-        .arg(service_pair.first)
+        .arg(server)
         .arg(model);
 
     auto callback = [=](Session session,
@@ -481,14 +488,14 @@ NetconfAgent::subscribeToNotifications(const CfgServersMapPair& service_pair) {
     };
     try {
         Subscription subscription(running_sess_->onNotification(model, callback));
-        subscriptions_.emplace(service_pair.first, forward<Subscription>(subscription));
-    } catch (const std::exception& ex) {
+        subscriptions_.emplace(server, forward<Subscription>(subscription));
+    } catch (exception const& ex) {
         ostringstream msg;
         msg << "event notification subscription for model " << model <<
             " failed with: " << ex.what();
-        LOG_ERROR(netconf_logger, NETCONF_SUBSCRIBE_NOTIFICATIONS_FAILED)
-            .arg(service_pair.first)
-            .arg(service_pair.second->getModel())
+        LOG_WARN(netconf_logger, NETCONF_NOT_SUBSCRIBED_TO_NOTIFICATIONS)
+            .arg(server)
+            .arg(configuration->getModel())
             .arg(msg.str());
         return;
     }
@@ -496,69 +503,72 @@ NetconfAgent::subscribeToNotifications(const CfgServersMapPair& service_pair) {
 
 sysrepo::ErrorCode
 NetconfAgent::change(Session sess, const CfgServersMapPair& service_pair) {
+    string const& server(service_pair.first);
+    CfgServerPtr const& configuration(service_pair.second);
+
     // If we're shutting down, or the subscribe-changes or the
     // validate-changes flag is not set or the model associated with
     // it is not specified.
-    if (!service_pair.second->getSubscribeChanges() ||
-        !service_pair.second->getValidateChanges() ||
-        service_pair.second->getModel().empty()) {
+    if (!configuration->getSubscribeChanges() ||
+        !configuration->getValidateChanges() ||
+        configuration->getModel().empty()) {
         return (sysrepo::ErrorCode::Ok);
     }
-    CfgControlSocketPtr ctrl_sock = service_pair.second->getCfgControlSocket();
+    CfgControlSocketPtr ctrl_sock = configuration->getCfgControlSocket();
     if (!ctrl_sock) {
         return (sysrepo::ErrorCode::Ok);
     }
     LOG_INFO(netconf_logger, NETCONF_VALIDATE_CONFIG_STARTED)
-        .arg(service_pair.first);
-    ConstElementPtr config;
+        .arg(server);
+    ElementPtr config;
     try {
-        TranslatorConfig tc(sess, service_pair.second->getModel());
+        TranslatorConfig tc(sess, configuration->getModel());
         config = tc.getConfig();
         if (!config) {
             ostringstream msg;
             msg << "YANG configuration for "
-                << service_pair.second->getModel()
+                << configuration->getModel()
                 << " is empty";
             LOG_ERROR(netconf_logger, NETCONF_VALIDATE_CONFIG_FAILED)
-                .arg(service_pair.first)
+                .arg(server)
                 .arg(msg.str());
             return (sysrepo::ErrorCode::OperationFailed);
         } else {
             LOG_DEBUG(netconf_logger, NETCONF_DBG_TRACE_DETAIL_DATA,
                       NETCONF_VALIDATE_CONFIG)
-                .arg(service_pair.first)
+                .arg(server)
                 .arg(prettyPrint(config));
         }
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         ostringstream msg;
-        msg << "get YANG configuration for " << service_pair.first
+        msg << "get YANG configuration for " << server
             << " failed with " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_VALIDATE_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
-        return (sysrepo::ErrorCode::ValidationFailed);;
+        return (sysrepo::ErrorCode::ValidationFailed);
     }
     ControlSocketBasePtr comm;
     try {
         comm = controlSocketFactory(ctrl_sock);
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         ostringstream msg;
         msg << "createControlSocket failed with " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_VALIDATE_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return (sysrepo::ErrorCode::Ok);
     }
     ConstElementPtr answer;
     int rcode;
     try {
-        answer = comm->configTest(config, service_pair.first);
+        answer = comm->configTest(config, server);
         parseAnswer(rcode, answer);
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         stringstream msg;
         msg << "configTest failed with " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_VALIDATE_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return (sysrepo::ErrorCode::ValidationFailed);
     }
@@ -566,23 +576,26 @@ NetconfAgent::change(Session sess, const CfgServersMapPair& service_pair) {
         stringstream msg;
         msg << "configTest returned " << answerToText(answer);
         LOG_ERROR(netconf_logger, NETCONF_VALIDATE_CONFIG_REJECTED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return (sysrepo::ErrorCode::ValidationFailed);
     }
     LOG_INFO(netconf_logger, NETCONF_VALIDATE_CONFIG_COMPLETED)
-        .arg(service_pair.first);
+        .arg(server);
     return (sysrepo::ErrorCode::Ok);
 }
 
 sysrepo::ErrorCode
 NetconfAgent::done(Session sess, const CfgServersMapPair& service_pair) {
+    string const& server(service_pair.first);
+    CfgServerPtr const& configuration(service_pair.second);
+
     // Check if we should and can process this update.
-    if (!service_pair.second->getSubscribeChanges() ||
-        service_pair.second->getModel().empty()) {
+    if (!configuration->getSubscribeChanges() ||
+        configuration->getModel().empty()) {
         return (sysrepo::ErrorCode::Ok);
     }
-    CfgControlSocketPtr ctrl_sock = service_pair.second->getCfgControlSocket();
+    CfgControlSocketPtr ctrl_sock = configuration->getCfgControlSocket();
     if (!ctrl_sock) {
         return (sysrepo::ErrorCode::Ok);
     }
@@ -590,34 +603,34 @@ NetconfAgent::done(Session sess, const CfgServersMapPair& service_pair) {
     // All looks good, let's get started. Print an info that we're about
     // to update the configuration.
     LOG_INFO(netconf_logger, NETCONF_UPDATE_CONFIG_STARTED)
-        .arg(service_pair.first);
+        .arg(server);
 
     // Retrieve the configuration from SYSREPO first.
-    ConstElementPtr config;
+    ElementPtr config;
     try {
-        TranslatorConfig tc(sess, service_pair.second->getModel());
+        TranslatorConfig tc(sess, configuration->getModel());
         config = tc.getConfig();
         if (!config) {
             ostringstream msg;
             msg << "YANG configuration for "
-                << service_pair.second->getModel()
+                << configuration->getModel()
                 << " is empty";
             LOG_ERROR(netconf_logger, NETCONF_UPDATE_CONFIG_FAILED)
-                .arg(service_pair.first)
+                .arg(server)
                 .arg(msg.str());
             return (sysrepo::ErrorCode::ValidationFailed);
         } else {
             LOG_DEBUG(netconf_logger, NETCONF_DBG_TRACE_DETAIL_DATA,
                       NETCONF_UPDATE_CONFIG)
-                .arg(service_pair.first)
+                .arg(server)
                 .arg(prettyPrint(config));
         }
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         ostringstream msg;
-        msg << "get YANG configuration for " << service_pair.first
+        msg << "get YANG configuration for " << server
             << " failed with " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_UPDATE_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return (sysrepo::ErrorCode::ValidationFailed);
     }
@@ -627,11 +640,11 @@ NetconfAgent::done(Session sess, const CfgServersMapPair& service_pair) {
     ControlSocketBasePtr comm;
     try {
         comm = controlSocketFactory(ctrl_sock);
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         ostringstream msg;
         msg << "createControlSocket failed with " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_UPDATE_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return (sysrepo::ErrorCode::Ok);
     }
@@ -640,13 +653,13 @@ NetconfAgent::done(Session sess, const CfgServersMapPair& service_pair) {
     ConstElementPtr answer;
     int rcode;
     try {
-        answer = comm->configSet(config, service_pair.first);
+        answer = comm->configSet(config, server);
         parseAnswer(rcode, answer);
-    } catch (const std::exception& ex) {
+    } catch (exception const& ex) {
         stringstream msg;
         msg << "configSet failed with " << ex.what();
         LOG_ERROR(netconf_logger, NETCONF_UPDATE_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return (sysrepo::ErrorCode::ValidationFailed);
     }
@@ -656,12 +669,12 @@ NetconfAgent::done(Session sess, const CfgServersMapPair& service_pair) {
         stringstream msg;
         msg << "configSet returned " << answerToText(answer);
         LOG_ERROR(netconf_logger, NETCONF_UPDATE_CONFIG_FAILED)
-            .arg(service_pair.first)
+            .arg(server)
             .arg(msg.str());
         return (sysrepo::ErrorCode::ValidationFailed);
     }
     LOG_INFO(netconf_logger, NETCONF_UPDATE_CONFIG_COMPLETED)
-        .arg(service_pair.first);
+        .arg(server);
     return (sysrepo::ErrorCode::Ok);
 }
 
@@ -669,7 +682,7 @@ void
 NetconfAgent::logChanges(Session sess, string_view const& model) {
     ostringstream stream;
     stream << "/" << model << ":*//.";
-    std::string const xpath(stream.str());
+    string const xpath(stream.str());
     ChangeCollection const changes(sess.getChanges(xpath));
     for (Change const& change : changes) {
         ostringstream msg;
@@ -698,7 +711,9 @@ NetconfAgent::logChanges(Session sess, string_view const& model) {
         } else if (node_type == NodeType::List) {
             msg << " (list)";
         } else {
-            optional<string> const str(TranslatorBasic::value(TranslatorBasic::value(change.node), LeafBaseType::Unknown));
+            optional<string> const str(
+                Translator::translateToYang(Translator::translateFromYang(change.node),
+                                            LeafBaseType::Unknown));
             if (str) {
                 msg << " = " << *str;
             }
@@ -726,5 +741,5 @@ bool NetconfAgent::shouldShutdown() const {
         ->shouldShutdown();
 }
 
-} // namespace netconf
-} // namespace isc
+}  // namespace netconf
+}  // namespace isc
