@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -346,6 +346,8 @@ const OptionDefParams STANDARD_V4_OPTION_DEFINITIONS[] = {
       OPT_IPV4_ADDRESS_TYPE, true, NO_RECORD_DEF, "" },
     { "sip-ua-cs-domains", DHO_SIP_UA_CONF_SERVICE_DOMAINS, DHCP4_OPTION_SPACE,
       OPT_FQDN_TYPE, true, NO_RECORD_DEF, "" },
+    { "v4-sztp-redirect", DHO_V4_SZTP_REDIRECT, DHCP4_OPTION_SPACE, OPT_TUPLE_TYPE,
+     true, NO_RECORD_DEF, ""},
     { "rdnss-selection", DHO_RDNSS_SELECT, DHCP4_OPTION_SPACE, OPT_RECORD_TYPE,
       true, RECORD_DEF(V4_RDNSS_SELECT_RECORDS), "" },
     { "status-code", DHO_STATUS_CODE, DHCP4_OPTION_SPACE,
@@ -638,6 +640,8 @@ const OptionDefParams STANDARD_V6_OPTION_DEFINITIONS[] = {
       OPT_STRING_TYPE, false, NO_RECORD_DEF, "" },
     { "relay-source-port", D6O_RELAY_SOURCE_PORT, DHCP6_OPTION_SPACE,
       OPT_UINT16_TYPE, false, NO_RECORD_DEF, "" },
+    { "v6-sztp-redirect", D60_V6_SZTP_REDIRECT, DHCP6_OPTION_SPACE,
+      OPT_TUPLE_TYPE, true, NO_RECORD_DEF, "" },
     { "ipv6-address-andsf", D6O_IPV6_ADDRESS_ANDSF, DHCP6_OPTION_SPACE,
       OPT_IPV6_ADDRESS_TYPE, true, NO_RECORD_DEF, "" },
     { "s46-cont-mape", D6O_S46_CONT_MAPE, DHCP6_OPTION_SPACE, OPT_EMPTY_TYPE,
