@@ -219,7 +219,7 @@ public:
           ddns_replace_client_name_mode_(), ddns_generated_prefix_(), ddns_qualifying_suffix_(),
           hostname_char_set_(), hostname_char_replacement_(), store_extended_info_(),
           cache_threshold_(), cache_max_age_(), ddns_update_on_renew_(),
-          ddns_use_conflict_resolution_() {
+          ddns_use_conflict_resolution_(), ddns_ttl_percent_() {
     }
 
     /// @brief Virtual destructor.
@@ -663,6 +663,23 @@ public:
         ddns_qualifying_suffix_ = ddns_qualifying_suffix;
     }
 
+    /// @brief Returns ddns-ttl-percent
+    ///
+    /// @param inheritance inheritance mode to be used.
+    util::Optional<double>
+    getDdnsTtlPercent(const Inheritance& inheritance = Inheritance::ALL) const {
+        return (getProperty<Network>(&Network::getDdnsTtlPercent,
+                                     ddns_ttl_percent_, inheritance,
+                                     CfgGlobals::DDNS_TTL_PERCENT));
+    }
+
+    /// @brief Sets new ddns-ttl-percent
+    ///
+    /// @param ddns_ttl_percent New value to use.
+    void setDdnsTtlPercent(const util::Optional<double>& ddns_ttl_percent) {
+        ddns_ttl_percent_ = ddns_ttl_percent;
+    }
+
     /// @brief Return the char set regexp used to sanitize client hostnames.
     util::Optional<std::string>
     getHostnameCharSet(const Inheritance& inheritance = Inheritance::ALL) const {
@@ -1195,6 +1212,9 @@ protected:
     /// @brief Allocator used for IP address allocations.
     util::Optional<std::string> allocator_type_;
 
+    /// @brief Percentage of the lease lifetime to use for DNS TTL.
+    util::Optional<double> ddns_ttl_percent_;
+
     /// @brief Pointer to another network that this network belongs to.
     ///
     /// The most common case is that this instance is a subnet which belongs
@@ -1215,7 +1235,7 @@ public:
     /// @brief Constructor.
     Network4()
         : Network(), match_client_id_(true, true), authoritative_(),
-          siaddr_(), sname_(), filename_() {
+          siaddr_(), sname_(), filename_(), offer_lft_() {
     }
 
     /// @brief Returns the flag indicating if the client identifiers should
@@ -1310,6 +1330,24 @@ public:
                                       CfgGlobals::BOOT_FILE_NAME));
     }
 
+    /// @brief Sets offer lifetime for the network.
+    ///
+    /// @param offer_lft the offer lifetime assigned to the class (may be empty if not defined)
+    void setOfferLft(const util::Optional<uint32_t>& offer_lft) {
+        offer_lft_ = offer_lft;
+    }
+
+    /// @brief Returns offer lifetime for the network
+    ///
+    /// @param inheritance inheritance mode to be used.
+    /// @return offer lifetime value
+    util::Optional<uint32_t>
+    getOfferLft(const Inheritance& inheritance = Inheritance::ALL) const {
+        return (getProperty<Network4>(&Network4::getOfferLft, offer_lft_,
+                                      inheritance,
+                                      CfgGlobals::OFFER_LIFETIME));
+    }
+
     /// @brief Unparses network object.
     ///
     /// @return A pointer to unparsed network configuration.
@@ -1330,16 +1368,22 @@ private:
     /// @brief Should requests for unknown IP addresses be rejected.
     util::Optional<bool> authoritative_;
 
-    /// @brief siaddr value for this subnet
+    /// @brief siaddr value for this network
     util::Optional<asiolink::IOAddress> siaddr_;
 
-    /// @brief server hostname for this subnet
+    /// @brief server hostname for this network
     util::Optional<std::string> sname_;
 
-    /// @brief boot file name for this subnet
+    /// @brief boot file name for this network
     util::Optional<std::string> filename_;
+
+    /// @brief offer lifetime for this network
+    util::Optional<uint32_t> offer_lft_;
 };
 
+/// @brief Pointer to the @ref Network4 object.
+typedef boost::shared_ptr<Network4> Network4Ptr;
+
 class Network6;
 
 /// @brief Pointer to the @ref Network6 object.
