@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2019-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -21,6 +21,7 @@
 #include <hooks/server_hooks.h>
 #include <hooks/callout_manager.h>
 #include <hooks/hooks_manager.h>
+#include <testutils/gtest_utils.h>
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/make_shared.hpp>
 #include <gtest/gtest.h>
@@ -307,13 +308,13 @@ public:
         auto& mgr = ConfigBackendDHCPv4Mgr::instance();
 
         // Insert global parameters into a database.
-        StampedValuePtr global_parameter = StampedValue::create("foo", "bar");
+        StampedValuePtr global_parameter = StampedValue::create("comment", "bar");
         global_parameter->setModificationTime(getTimestamp("dhcp4_global_parameter"));
         ASSERT_NO_THROW(mgr.getPool()->createUpdateGlobalParameter4(BackendSelector::UNSPEC(),
                                                                     ServerSelector::ALL(),
                                                                     global_parameter));
 
-        global_parameter = StampedValue::create("bar", "teta");
+        global_parameter = StampedValue::create("next-server", "teta");
         global_parameter->setModificationTime(getTimestamp("dhcp4_global_parameter"));
         ASSERT_NO_THROW(mgr.getPool()->createUpdateGlobalParameter4(BackendSelector::UNSPEC(),
                                                                     ServerSelector::ALL(),
@@ -336,7 +337,8 @@ public:
         // Insert global options into the database.
         OptionDescriptorPtr opt(new OptionDescriptor(createOption<OptionString>
                                                      (Option::V4, DHO_HOST_NAME,
-                                                      true, false, "new.example.com")));
+                                                      true, false, false,
+                                                      "new.example.com")));
         opt->setId(1);
         opt->space_name_ = DHCP4_OPTION_SPACE;
         opt->setModificationTime(getTimestamp("dhcp4_options"));
@@ -345,7 +347,7 @@ public:
 
         opt.reset(new OptionDescriptor(createOption<OptionString>
                                        (Option::V4, DHO_TFTP_SERVER_NAME,
-                                        true, false, "tftp-my")));
+                                        true, false, false, "tftp-my")));
         opt->setId(2);
         opt->space_name_ = DHCP4_OPTION_SPACE;
         opt->setModificationTime(getTimestamp("dhcp4_options"));
@@ -385,6 +387,32 @@ public:
         client_class->setTest("substring(option[1].hex, 0, 8) == 'my-value'");
         client_class->setId(1);
         client_class->setModificationTime(getTimestamp("dhcp4_client_class"));
+
+        // Add a standard option to the class.
+        OptionPtr option = Option::create(Option::V4, DHO_BOOT_FILE_NAME);
+        OptionDescriptorPtr desc = OptionDescriptor::create(option,
+                                                            true,
+                                                            false,
+                                                            "bogus-file.txt");
+        desc->space_name_ = DHCP4_OPTION_SPACE;
+        desc->setModificationTime(getTimestamp("dhcp4_client_class"));
+        client_class->getCfgOption()->add(*desc, desc->space_name_);
+
+        // Add a custom option definition to the class.
+        CfgOptionDefPtr cc_cfg_option_def(new CfgOptionDef());
+        def.reset(new OptionDefinition("v4str", 201, "isc", "string"));
+        def->setId(201);
+        def->setModificationTime(getTimestamp("dhcp4_client_class"));
+        cc_cfg_option_def->add(def);
+        client_class->setCfgOptionDef(cc_cfg_option_def);
+
+        // Add a custom option to the class.
+        option = Option::create(Option::V4, 201);
+        desc = OptionDescriptor::create(option, true, false, "custom-stuff");
+        desc->space_name_ = "isc";
+        desc->setModificationTime(getTimestamp("dhcp4_client_class"));
+        client_class->getCfgOption()->add(*desc, desc->space_name_);
+
         mgr.getPool()->createUpdateClientClass4(BackendSelector::UNSPEC(), ServerSelector::ALL(),
                                                 client_class, "");
 
@@ -515,8 +543,8 @@ public:
         ASSERT_FALSE(audit_entries_.empty())
             << "Require at least one audit entry. The test is broken!";
 
-        ctl_.databaseConfigApply(BackendSelector::UNSPEC(), ServerSelector::ALL(),
-                                 lb_modification_time, audit_entries_);
+        ASSERT_NO_THROW_LOG(ctl_.databaseConfigApply(BackendSelector::UNSPEC(), ServerSelector::ALL(),
+                            lb_modification_time, audit_entries_));
 
         // The updates should have been merged into current configuration.
         auto srv_cfg = CfgMgr::instance().getCurrentCfg();
@@ -526,11 +554,11 @@ public:
         // be merged.
         if (hasConfigElement("dhcp4_global_parameter") &&
             (getTimestamp("dhcp4_global_parameter") > lb_modification_time)) {
-            checkConfiguredGlobal(srv_cfg, "foo", Element::create("bar"));
+            checkConfiguredGlobal(srv_cfg, "comment", Element::create("bar"));
 
         } else {
             // Otherwise it shouldn't exist.
-            EXPECT_FALSE(srv_cfg->getConfiguredGlobals()->get("foo"));
+            EXPECT_FALSE(srv_cfg->getConfiguredGlobals()->get("comment"));
         }
 
         // If there is an audit entry for option definition and the definition
@@ -578,7 +606,7 @@ public:
         // If there is an audit entry for a subnet and the subnet modification
         // time is later than last audit revision time it should be merged.
         auto subnets = srv_cfg->getCfgSubnets4();
-        auto found_subnet = subnets->getSubnet(1);
+        auto found_subnet = subnets->getBySubnetId(1);
         if (hasConfigElement("dhcp4_subnet") &&
             (getTimestamp("dhcp4_subnet") > lb_modification_time)) {
             ASSERT_TRUE(found_subnet);
@@ -597,6 +625,23 @@ public:
             EXPECT_GT(found_class->getMatchExpr()->size(), 0);
             EXPECT_EQ("first-class", found_class->getName());
 
+            // Check for the standard class option, make sure it has been "created".
+            auto cfg_option_desc = found_class->getCfgOption();
+            ASSERT_TRUE(cfg_option_desc);
+            auto option_desc = cfg_option_desc->get(DHCP4_OPTION_SPACE, DHO_BOOT_FILE_NAME);
+            auto option = option_desc.option_;
+            ASSERT_TRUE(option);
+            EXPECT_EQ(OptionBuffer(option_desc.formatted_value_.begin(),
+                                   option_desc.formatted_value_.end()),
+                      option->getData());
+
+            // Check for the custom class option, make sure it has been "created".
+            option_desc = cfg_option_desc->get("isc", 201);
+            option = option_desc.option_;
+            ASSERT_TRUE(option);
+            EXPECT_EQ(OptionBuffer(option_desc.formatted_value_.begin(),
+                                   option_desc.formatted_value_.end()),
+                      option->getData());
         } else {
             EXPECT_FALSE(found_class);
         }
@@ -653,12 +698,12 @@ public:
         {
             SCOPED_TRACE("global parameters");
             // One of the global parameters should still be there.
-            EXPECT_TRUE(srv_cfg->getConfiguredGlobals()->get("bar"));
+            EXPECT_TRUE(srv_cfg->getConfiguredGlobals()->get("next-server"));
             if (deleteConfigElement("dhcp4_global_parameter", 1)) {
-                EXPECT_FALSE(srv_cfg->getConfiguredGlobals()->get("foo"));
+                EXPECT_FALSE(srv_cfg->getConfiguredGlobals()->get("comment"));
 
             } else {
-                EXPECT_TRUE(srv_cfg->getConfiguredGlobals()->get("foo"));
+                EXPECT_TRUE(srv_cfg->getConfiguredGlobals()->get("next-server"));
             }
         }
 
@@ -706,8 +751,8 @@ public:
         {
             SCOPED_TRACE("subnets");
             // One of the subnets should still be there.
-            EXPECT_TRUE(srv_cfg->getCfgSubnets4()->getSubnet(2));
-            auto found_subnet = srv_cfg->getCfgSubnets4()->getSubnet(1);
+            EXPECT_TRUE(srv_cfg->getCfgSubnets4()->getBySubnetId(2));
+            auto found_subnet = srv_cfg->getCfgSubnets4()->getBySubnetId(1);
             if (deleteConfigElement("dhcp4_subnet", 1)) {
                 EXPECT_FALSE(found_subnet);
 
@@ -768,7 +813,7 @@ TEST_F(CBControlDHCPv4Test, databaseConfigApplyAll) {
 // deleted from the database.
 TEST_F(CBControlDHCPv4Test, databaseConfigApplyDeleteAll) {
     testDatabaseConfigApplyDelete(getTimestamp(-5), [this]() {
-        remoteDeleteGlobalParameter("foo", 1);
+        remoteDeleteGlobalParameter("comment", 1);
         remoteDeleteOptionDef(101, "isc");
         remoteDeleteOption(DHO_HOST_NAME, DHCP4_OPTION_SPACE);
         remoteDeleteSharedNetwork("one");
@@ -807,7 +852,7 @@ TEST_F(CBControlDHCPv4Test, databaseConfigApplyGlobal) {
 // database.
 TEST_F(CBControlDHCPv4Test, databaseConfigApplyDeleteGlobal) {
     testDatabaseConfigApplyDelete(getTimestamp(-5), [this]() {
-        remoteDeleteGlobalParameter("foo", 1);
+        remoteDeleteGlobalParameter("comment", 1);
     });
 }
 
@@ -1103,13 +1148,13 @@ public:
         auto& mgr = ConfigBackendDHCPv6Mgr::instance();
 
         // Insert global parameters into a database.
-        StampedValuePtr global_parameter = StampedValue::create("foo", "bar");
+        StampedValuePtr global_parameter = StampedValue::create("comment", "bar");
         global_parameter->setModificationTime(getTimestamp("dhcp6_global_parameter"));
         ASSERT_NO_THROW(mgr.getPool()->createUpdateGlobalParameter6(BackendSelector::UNSPEC(),
                                                                     ServerSelector::ALL(),
                                                                     global_parameter));
 
-        global_parameter = StampedValue::create("bar", "teta");
+        global_parameter = StampedValue::create("data-directory", "teta");
         global_parameter->setModificationTime(getTimestamp("dhcp6_global_parameter"));
         ASSERT_NO_THROW(mgr.getPool()->createUpdateGlobalParameter6(BackendSelector::UNSPEC(),
                                                                     ServerSelector::ALL(),
@@ -1132,7 +1177,8 @@ public:
         // Insert global options into the database.
         OptionDescriptorPtr opt(new OptionDescriptor(createOption<OptionString>
                                                      (Option::V6, D6O_BOOTFILE_URL,
-                                                      true, false, "some.bootfile")));
+                                                      true, false, false,
+                                                      "some.bootfile")));
         opt->setId(1);
         opt->space_name_ = DHCP6_OPTION_SPACE;
         opt->setModificationTime(getTimestamp("dhcp6_options"));
@@ -1141,7 +1187,8 @@ public:
 
         opt.reset(new OptionDescriptor(createOption<OptionString>
                                        (Option::V6, D6O_AFTR_NAME,
-                                        true, true, "some.example.com")));
+                                        true, true, true,
+                                        "some.example.com")));
         opt->setId(2);
         opt->space_name_ = DHCP6_OPTION_SPACE;
         opt->setModificationTime(getTimestamp("dhcp6_options"));
@@ -1181,6 +1228,16 @@ public:
         client_class->setTest("substring(option[1].hex, 0, 8) == 'my-value'");
         client_class->setId(1);
         client_class->setModificationTime(getTimestamp("dhcp6_client_class"));
+
+        // Add an option to the class.
+        OptionPtr option = Option::create(Option::V6, D6O_BOOTFILE_URL);
+        OptionDescriptorPtr desc = OptionDescriptor::create(option, true,
+                                                            false,
+                                                            "client.boot.url");
+        desc->space_name_ = DHCP6_OPTION_SPACE;
+        desc->setModificationTime(getTimestamp("dhcp6_client_class"));
+        client_class->getCfgOption()->add(*desc, desc->space_name_);
+
         mgr.getPool()->createUpdateClientClass6(BackendSelector::UNSPEC(), ServerSelector::ALL(),
                                                 client_class, "");
 
@@ -1322,11 +1379,11 @@ public:
         // be merged.
         if (hasConfigElement("dhcp6_global_parameter") &&
             (getTimestamp("dhcp6_global_parameter") > lb_modification_time)) {
-            checkConfiguredGlobal(srv_cfg, "foo", Element::create("bar"));
+            checkConfiguredGlobal(srv_cfg, "comment", Element::create("bar"));
 
         } else {
             // Otherwise it shouldn't exist.
-            EXPECT_FALSE(srv_cfg->getConfiguredGlobals()->get("foo"));
+            EXPECT_FALSE(srv_cfg->getConfiguredGlobals()->get("comment"));
         }
 
         // If there is an audit entry for option definition and the definition
@@ -1374,7 +1431,7 @@ public:
         // If there is an audit entry for a subnet and the subnet modification
         // time is later than last audit revision time it should be merged.
         auto subnets = srv_cfg->getCfgSubnets6();
-        auto found_subnet = subnets->getSubnet(1);
+        auto found_subnet = subnets->getBySubnetId(1);
         if (hasConfigElement("dhcp6_subnet") &&
             (getTimestamp("dhcp6_subnet") > lb_modification_time)) {
             ASSERT_TRUE(found_subnet);
@@ -1393,6 +1450,15 @@ public:
             EXPECT_GT(found_class->getMatchExpr()->size(), 0);
             EXPECT_EQ("first-class", found_class->getName());
 
+            // Check for class option, make sure it has been "created".
+            auto cfg_option_desc = found_class->getCfgOption();
+            ASSERT_TRUE(cfg_option_desc);
+            auto option_desc = cfg_option_desc->get(DHCP6_OPTION_SPACE, D6O_BOOTFILE_URL);
+            auto option = option_desc.option_;
+            ASSERT_TRUE(option);
+            EXPECT_EQ(OptionBuffer(option_desc.formatted_value_.begin(),
+                                   option_desc.formatted_value_.end()),
+                      option->getData());
         } else {
             EXPECT_FALSE(found_class);
         }
@@ -1449,12 +1515,12 @@ public:
         {
             SCOPED_TRACE("global parameters");
             // One of the global parameters should still be there.
-            EXPECT_TRUE(srv_cfg->getConfiguredGlobals()->get("bar"));
+            EXPECT_TRUE(srv_cfg->getConfiguredGlobals()->get("data-directory"));
             if (deleteConfigElement("dhcp6_global_parameter", 1)) {
-                EXPECT_FALSE(srv_cfg->getConfiguredGlobals()->get("foo"));
+                EXPECT_FALSE(srv_cfg->getConfiguredGlobals()->get("comment"));
 
             } else {
-                EXPECT_TRUE(srv_cfg->getConfiguredGlobals()->get("foo"));
+                EXPECT_TRUE(srv_cfg->getConfiguredGlobals()->get("comment"));
             }
         }
 
@@ -1502,8 +1568,8 @@ public:
         {
             SCOPED_TRACE("subnets");
             // One of the subnets should still be there.
-            EXPECT_TRUE(srv_cfg->getCfgSubnets6()->getSubnet(2));
-            auto found_subnet = srv_cfg->getCfgSubnets6()->getSubnet(1);
+            EXPECT_TRUE(srv_cfg->getCfgSubnets6()->getBySubnetId(2));
+            auto found_subnet = srv_cfg->getCfgSubnets6()->getBySubnetId(1);
             if (deleteConfigElement("dhcp6_subnet", 1)) {
                 EXPECT_FALSE(found_subnet);
                 // If the subnet has been deleted, make sure that
@@ -1550,7 +1616,7 @@ TEST_F(CBControlDHCPv6Test, databaseConfigApplyAll) {
 // deleted from the database.
 TEST_F(CBControlDHCPv6Test, databaseConfigApplyDeleteAll) {
     testDatabaseConfigApplyDelete(getTimestamp(-5), [this]() {
-        remoteDeleteGlobalParameter("foo", 1);
+        remoteDeleteGlobalParameter("comment", 1);
         remoteDeleteOptionDef(101, "isc");
         remoteDeleteOption(D6O_BOOTFILE_URL, DHCP6_OPTION_SPACE);
         remoteDeleteSharedNetwork("one");
@@ -1587,7 +1653,7 @@ TEST_F(CBControlDHCPv6Test, databaseConfigApplyGlobal) {
 // database.
 TEST_F(CBControlDHCPv6Test, databaseConfigApplyDeleteGlobal) {
     testDatabaseConfigApplyDelete(getTimestamp(-5), [this]() {
-        remoteDeleteGlobalParameter("foo", 1);
+        remoteDeleteGlobalParameter("comment", 1);
     });
 }
 
