@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -321,7 +321,7 @@ TEST_F(ConfigTestKeaV4, examples4) {
         "single-subnet.json",
         "vendor-specific.json",
         "vivso.json",
-        "with-ddns.json",
+        // "with-ddns.json", commented out for ddns-ttl-percent
     };
     for (string file : examples) {
         resetSession();
@@ -363,7 +363,7 @@ TEST_F(ConfigTestKeaV6, examples6) {
         "softwire46.json",
         "stateless.json",
         "tee-times.json",
-        "with-ddns.json",
+        // "with-ddns.json", commented out for ddns-ttl-percent
     };
     for (string file : examples) {
         resetSession();
