@@ -7,8 +7,8 @@
 #include <config.h>
 
 #include <yang/translator_database.h>
-#include <yang/adaptor.h>
 #include <yang/yang_models.h>
+
 #include <sstream>
 
 using namespace std;
@@ -20,10 +20,7 @@ namespace isc {
 namespace yang {
 
 TranslatorDatabase::TranslatorDatabase(Session session, const string& model)
-    : TranslatorBasic(session, model) {
-}
-
-TranslatorDatabase::~TranslatorDatabase() {
+    : Translator(session, model) {
 }
 
 ElementPtr
@@ -34,52 +31,49 @@ TranslatorDatabase::getDatabase(DataNode const& data_node) {
             return (getDatabaseKea(data_node));
         }
     } catch (Error const& ex) {
-        isc_throw(SysrepoError,
-                  "sysrepo error getting database access: " << ex.what());
+        isc_throw(NetconfError,
+                  "getting database access: " << ex.what());
     }
     isc_throw(NotImplemented,
               "getDatabase not implemented for the model: " << model_);
 }
 
 ElementPtr
-TranslatorDatabase::getDatabase(string const& xpath) {
+TranslatorDatabase::getDatabaseFromAbsoluteXpath(string const& xpath) {
     try {
         return getDatabase(findXPath(xpath));
-    } catch(SysrepoError const&) {
+    } catch (NetconfError const&) {
         return ElementPtr();
     }
 }
 
 ElementPtr
 TranslatorDatabase::getDatabaseKea(DataNode const& data_node) {
-    ConstElementPtr type = getItem(data_node, "database-type");
-    if (!type) {
-        return (ElementPtr());
-    }
     ElementPtr result = Element::createMap();
-    result->set("type", type);
-    checkAndGetLeaf(result, data_node, "user");
-    checkAndGetLeaf(result, data_node, "password");
-    checkAndGetLeaf(result, data_node, "host");
-    checkAndGetLeaf(result, data_node, "name");
-    checkAndGetLeaf(result, data_node, "persist");
-    checkAndGetLeaf(result, data_node, "port");
-    checkAndGetLeaf(result, data_node, "lfc-interval");
-    checkAndGetLeaf(result, data_node, "readonly");
-    checkAndGetLeaf(result, data_node, "trust-anchor");
+
+    getMandatoryDivergingLeaf(result, data_node, "type", "database-type");
+
     checkAndGetLeaf(result, data_node, "cert-file");
-    checkAndGetLeaf(result, data_node, "key-file");
     checkAndGetLeaf(result, data_node, "cipher-list");
     checkAndGetLeaf(result, data_node, "connect-timeout");
+    checkAndGetLeaf(result, data_node, "host");
+    checkAndGetLeaf(result, data_node, "key-file");
+    checkAndGetLeaf(result, data_node, "lfc-interval");
     checkAndGetLeaf(result, data_node, "max-reconnect-tries");
-    checkAndGetLeaf(result, data_node, "reconnect-wait-time");
     checkAndGetLeaf(result, data_node, "max-row-errors");
+    checkAndGetLeaf(result, data_node, "name");
     checkAndGetLeaf(result, data_node, "on-fail");
-    ConstElementPtr context = getItem(data_node, "user-context");
-    if (context) {
-        result->set("user-context", Element::fromJSON(context->stringValue()));
-    }
-    return (result);
+    checkAndGetLeaf(result, data_node, "password");
+    checkAndGetLeaf(result, data_node, "persist");
+    checkAndGetLeaf(result, data_node, "port");
+    checkAndGetLeaf(result, data_node, "readonly");
+    checkAndGetLeaf(result, data_node, "reconnect-wait-time");
+    checkAndGetLeaf(result, data_node, "trust-anchor");
+    checkAndGetLeaf(result, data_node, "user");
+
+    checkAndGetAndJsonifyLeaf(result, data_node, "user-context");
+
+    return (result->empty() ? ElementPtr() : result);
 }
 
 void
@@ -95,8 +89,8 @@ TranslatorDatabase::setDatabase(string const& xpath,
                       "setDatabase not implemented for the model: " << model_);
         }
     } catch (Error const& ex) {
-        isc_throw(SysrepoError,
-                  "sysrepo error setting database access '" << elem->str()
+        isc_throw(NetconfError,
+                  "setting database access '" << elem->str()
                   << "' : " << ex.what());
     }
 }
@@ -106,50 +100,41 @@ TranslatorDatabase::setDatabaseKea(string const& xpath,
                                    ConstElementPtr elem,
                                    bool skip) {
     if (!elem) {
-        delItem(xpath);
+        deleteItem(xpath);
         return;
     }
-    if (!skip) {
-        ConstElementPtr type = elem->get("type");
-        if (!type) {
-            isc_throw(BadValue, "setDatabase requires database type: "
-                      << elem->str());
-        }
-        setItem(xpath + "/database-type", type, LeafBaseType::String);
-    }
-    checkAndSetLeaf(elem, xpath, "user", LeafBaseType::String);
-    checkAndSetLeaf(elem, xpath, "password", LeafBaseType::String);
+
+    checkAndSetLeaf(elem, xpath, "connect-timeout", LeafBaseType::Uint32);
+    checkAndSetLeaf(elem, xpath, "cert-file", LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "cipher-list", LeafBaseType::String);
     checkAndSetLeaf(elem, xpath, "host", LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "key-file", LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "lfc-interval", LeafBaseType::Uint32);
+    checkAndSetLeaf(elem, xpath, "max-reconnect-tries", LeafBaseType::Uint32);
+    checkAndSetLeaf(elem, xpath, "max-row-errors", LeafBaseType::Uint32);
     checkAndSetLeaf(elem, xpath, "name", LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "on-fail", LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "password", LeafBaseType::String);
     checkAndSetLeaf(elem, xpath, "persist", LeafBaseType::Bool);
     checkAndSetLeaf(elem, xpath, "port", LeafBaseType::Uint16);
-    checkAndSetLeaf(elem, xpath, "lfc-interval", LeafBaseType::Uint32);
     checkAndSetLeaf(elem, xpath, "readonly", LeafBaseType::Bool);
-    checkAndSetLeaf(elem, xpath, "trust-anchor", LeafBaseType::String);
-    checkAndSetLeaf(elem, xpath, "cert-file", LeafBaseType::String);
-    checkAndSetLeaf(elem, xpath, "key-file", LeafBaseType::String);
-    checkAndSetLeaf(elem, xpath, "cipher-list", LeafBaseType::String);
-    checkAndSetLeaf(elem, xpath, "connect-timeout", LeafBaseType::Uint32);
-    checkAndSetLeaf(elem, xpath, "max-reconnect-tries", LeafBaseType::Uint32);
     checkAndSetLeaf(elem, xpath, "reconnect-wait-time", LeafBaseType::Uint32);
-    checkAndSetLeaf(elem, xpath, "max-row-errors", LeafBaseType::Uint32);
-    checkAndSetLeaf(elem, xpath, "on-fail", LeafBaseType::String);
-    ConstElementPtr context = Adaptor::getContext(elem);
-    if (context) {
-        setItem(xpath + "/user-context", Element::create(context->str()),
-                LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "trust-anchor", LeafBaseType::String);
+    checkAndSetLeaf(elem, xpath, "user", LeafBaseType::String);
+
+    checkAndSetUserContext(elem, xpath);
+
+    if (!skip) {
+        setMandatoryDivergingLeaf(elem, xpath, "type", "database-type", LeafBaseType::String);
     }
 }
 
 TranslatorDatabases::TranslatorDatabases(Session session,
                                          const string& model)
-    : TranslatorBasic(session, model),
+    : Translator(session, model),
       TranslatorDatabase(session, model) {
 }
 
-TranslatorDatabases::~TranslatorDatabases() {
-}
-
 ElementPtr
 TranslatorDatabases::getDatabases(DataNode const& data_node,
                                   string const& xpath) {
@@ -159,24 +144,24 @@ TranslatorDatabases::getDatabases(DataNode const& data_node,
             return (getDatabasesKea(data_node, xpath));
         }
     } catch (Error const& ex) {
-        isc_throw(SysrepoError,
-                  "sysrepo error getting database accesses: " << ex.what());
+        isc_throw(NetconfError,
+                  "getting database accesses: " << ex.what());
     }
     isc_throw(NotImplemented,
               "getDatabases not implemented for the model: " << model_);
 }
 
 ElementPtr
-TranslatorDatabases::getDatabases(string const& xpath) {
+TranslatorDatabases::getDatabasesFromAbsoluteXpath(string const& xpath) {
     try {
         return getDatabases(findXPath(xpath), xpath);
-    } catch(SysrepoError const&) {
+    } catch (NetconfError const&) {
         return ElementPtr();
     }
 }
 
 ElementPtr
-TranslatorDatabases::getDatabasesKea(DataNode const& data_node, std::string const& xpath) {
+TranslatorDatabases::getDatabasesKea(DataNode const& data_node, string const& xpath) {
     return getList<TranslatorDatabase>(data_node, xpath, *this,
                                        &TranslatorDatabase::getDatabase);
 }
@@ -193,8 +178,8 @@ TranslatorDatabases::setDatabases(string const& xpath, ConstElementPtr elem) {
                       << model_);
         }
     } catch (Error const& ex) {
-        isc_throw(SysrepoError,
-                  "sysrepo error setting database accesses '" << elem->str()
+        isc_throw(NetconfError,
+                  "setting database accesses '" << elem->str()
                   << "' : " << ex.what());
     }
 }
@@ -203,11 +188,11 @@ void
 TranslatorDatabases::setDatabasesKea(string const& xpath,
                                      ConstElementPtr elem) {
     if (!elem) {
-        delItem(xpath);
+        deleteItem(xpath);
         return;
     }
     for (size_t i = 0; i < elem->size(); ++i) {
-        ConstElementPtr database = elem->get(i);
+        ElementPtr database = elem->getNonConst(i);
         if (!database->contains("type")) {
             isc_throw(BadValue, "database without type: " << database->str());
         }
